#include "stdafx.h"
#include "Cchess_1.h"
#include "Cchess_2.h"
//#include "Cchess_3.h"
#include "Cchess_4.h"
#include "Cchess_5.h"
#include "Cchess_6.h"



/* Gibbon 2.60
En extend : 
0 - Hint_move control
1 - Prises = boucle_pice actuelle
    jouer toutes les prises si chec
2 - Prises_pions : idem
    jouer toutes les prises si chec
3 - Echecs slectionns sans checs : liste gnre de coups contrls
    avec les listes de coups par pice
    => on fait un create liste avant de jouer les checs
    => partir des cases d'actualisation
    => obtenir une liste exhaustive pour F_extension_echecs
       (au moins les checs de pion)
3bis - Autres coups sous chec => partir de la liste (ce qu'on fait dja aujourd'hui)
=> on est toujours en analyse liste

En search :
1 - Prises
2 - Echecs en cration liste
    avec maj dep_arr[x][y]
3 - Autres coups en cration liste
    Pas de rduction late coups heuristiques (checs rsiduels, refutation_departure ...)
    

Conclusion :
- on conserve l'examen des prises en mode hors liste pour cutoff rapide
- on n'utilise plus de fonction givecheck dans search et extend
  => gain de temps
  car on n'obtient les checs via les int64
- l'analyse des autres coups sous echec se fait sous liste
- en search, on supprime la boucle sur les coups heuristiques = gain de temps,
  remplace par une boucle sur les checs favorables

Ides 
- une autre routine de validation de coup
  que l'ide consistant  rechercher le coup
  dans la liste de coups
  A - vrif nature move et xboard[dep]
  B - vrif AllPieces et masque[dep][arr]
  C - vrif absence de clouage
  D - vrif prise
  E - vrif cohrence checs
  F - coups spciaux
      vrification de lgalit
*/

  
 


void SEARCH::extend_initialize_move_generator()
{
char dep, arr;
char cc,dd;
char move_nature;



F_hint_move_has_been_played=false;
F_utilisation_liste=false;
nb_prises_favorables=0;
nb_prises_pion_non_defendus=0;
nb_echanges_piece=0;
nb_echanges_pion=0;
nb_prises_piece_inferieure=0;
nb_prises_insuffisantes_candidates_echec=0;
#if EXTEND_USE_CASES_ACTUALISATION_FOR_CHECKS==1
nb_echecs=0;
#endif
nb_autres_coups=0;
nb_coups_declasses=0;
nb_coups_liste=0;
F_pas_de_prises=false;
MOVE move;


#if EXTENDLEVEL_KILLERMOVE==1
if (hint_move==0)
{
	if (xlevel==xextendlevel)
	{
		if (previous_search_level->nbplayed!=1)
		{
			move=extend_oldbest;
			if (MOVE_IS_FORCING(move))
			{
				if (xboard[MOVE_ARR64(move)].nature!=0)
				{
					if (xboard[MOVE_DEP64(move)].nature==MOVE_NATURE(move))
					{
						hint_move=move;
					}
				}
			}
		}
	}
}
#endif


// Contrle de lgalit du hint_move
// et insertion dans liste hint_move
if (hint_move!=0) 
{
if (MOVE_IS_FOR_PLAY(hint_move))
{
dd=0;
do
{
	dep=MOVE_DEP64(hint_move);
	arr=MOVE_ARR64(hint_move);
	num=xboard[dep].num_piece;
	if (num<debut_blanc || num>fin_blanc)
	{
		#if DEBUG==1
		{
			log_error_no_stop("EM_814", xcompteur);
		}
		#endif
		break;
    }
	numptr=&xw[num];
	if (numptr->nature==0) break;
	if (MOVE_CAPTURE(hint_move)!=0 || MOVE_PROMOTION(hint_move)=='D')
	{
		moveptr=numptr->psptr->moves2;
	}
	else
	{
		moveptr=numptr->psptr->moves;
	}
	for(;;moveptr++)
	{
		move=*moveptr;
		if (move==0) break;
		
		if (SAME_MOVE(move,hint_move))
		{
			// Eviter les suites infinies d'checs
			if (!MOVE_IS_CAPTURE(move) && !MOVE_IS_PROMOTION(move) 
				#if USE_MATE_IN_ONE==1
				&& !F_case_de_mat
				#endif
				)
			{
				// Si au dela des checs, pas d'checs !
				if (xlevel>levelmax_echecs)
				{ break; }
				// Le coup donne-t-il chec, d'ailleurs ?
				// Vrification sommaire
				move_nature=MOVE_NATURE(move);
				cc=0;
				switch(move_nature)
				{
				case 'P' : if (givecheck_pion(MOVE_ARR64(move),wtm))
						   { cc=1; }
						   break;
				case 'C' : if (givecheck_cavalier(MOVE_ARR64(move)))
						   { cc=1; }
						   break;
				case 'F' : if (givecheck64(s->cases_echec_fou[wtm],MOVE_ARR64(move)))
						   { cc=1; }
						   break;
				case 'T' : if (givecheck64(s->cases_echec_tour[wtm],MOVE_ARR64(move))) 
						   { cc=1; }
						   break;
				case 'D' : if (givecheck64(s->cases_echec_dame[wtm],MOVE_ARR64(move)))
						   { cc=1; }
						   break;
				case 'R' : if (givecheck_roi(move))
						   { cc=1; }
				default : break;
				}
				if (cc!=1) break;
			}

			//legality=(LEGALITY) 0;		
			// Contrle lgalit
			#if USE_CASES_ACTUALISATION==1
			if (bit(pieces_with_possible_illegal_moves,numptr->num_piece))
			#endif
			{
				cc=SEARCH::legal(num,move,num_roi,*lignemoins);
				if (cc!=1) break;
			}

			dd=1; // hint_move est lgal : IMPORTANT
			nb_coups_liste=1;
			hint_moves[0]=move; // pas hint_move qui peut-tre uen capture
			hint_moves[1]=0;
			
			//listmovesptr->legal=LEGAL;
			//listmovesptr->move=move;
			//*listmovesptr=move;
			//listmovesptr->move_nature=MOVE_NATURE(move);
			//listmovesptr->num=num;
			//listmovesptr->categorie=(char) MOVE_CATEGORY(move);
			//listmovesptr->capture_value=MOVE_CAPTURE_VALUE(move);
			//listmovesptr->Fplay=0;
			//listmovesptr->bilan_coup=BILAN_NON_CALCULE;
			//listmovesptr->hash=HASH_NON_DETERMINE;
			//listmovesptr->draw_status=DRAW_STATUS_NOT_DETERMINED;
			break;
		}
	} 
	break;
} while(1);
// Annulation du hint_move si illgal
if (dd==0) 
{
	hint_move=0;
}
}
} // fin hint_move!=0


if (check!=0)
{
	// Cases de couverture
      // cases_couverture_echecs=xsearch_level[xlevel].couverture_echec_64;
	extend_cree_liste_prises();
	extend_cree_liste_autres_coups();
	if (!F_existe_coup_legal) return;
	phase_generation=AUTRES_COUPS;
	F_utilisation_liste=true;
	if (hint_move!=0) 
	{
		analysis=EXTEND_ANALYSE_HINT_MOVE;
	    #if COLLECT_STATS==1
	    origine_coups_analysis_extend=OR_EXTEND_ANALYSE_HINT_HMOVE;
	    #endif
		moveptr=&hint_moves[0]-1;
	}
	else 
	if (nb_prises_favorables>0)
	{
		analysis=EXTEND_ANALYSE_COUPS_FAVORABLES;
	    #if COLLECT_STATS==1
	    origine_coups_analysis_extend=OR_EXTEND_ANALYSE_COUPS_FAVORABLES;
	    #endif
		moveptr=&prises_favorables[0]-1;
	}
	else if (nb_echanges_piece>0)
	{
		analysis=EXTEND_ANALYSE_ECHANGES_PIECE;
	    #if COLLECT_STATS==1
	    origine_coups_analysis_extend=OR_EXTEND_ANALYSE_ECHANGE_PIECE;
	    #endif
		moveptr=&echanges_piece[0]-1;
		echanges_piece[nb_echanges_piece]=0;
	}
	else if (nb_prises_pion_non_defendus>0)
	{
		analysis=EXTEND_PRISES_PIONS_NON_DEFENDUS;
	    #if COLLECT_STATS==1
	    origine_coups_analysis_extend=OR_EXTEND_PRISES_PIONS_NON_DEFENDUS;
	    #endif
		moveptr=&prises_pion_non_defendus[0]-1;
		prises_pion_non_defendus[nb_prises_pion_non_defendus]=0;
	}
	else if (nb_prises_piece_inferieure>0)
	{
		analysis=EXTEND_PRISES_PIECE_INFERIEURE;
	    #if COLLECT_STATS==1
	    origine_coups_analysis_extend=OR_EXTEND_PRISE_PIECE_INFERIEURE;
	    #endif
		moveptr=&prises_piece_inferieure[0]-1;
		prises_piece_inferieure[nb_prises_piece_inferieure]=0;
	}
	else if (nb_echanges_pion>0)
	{
		analysis=EXTEND_ANALYSE_ECHANGES_PION;
	    #if COLLECT_STATS==1
	    origine_coups_analysis_extend=OR_EXTEND_ANALYSE_ECHANGE_PION;
	    #endif
		moveptr=&echanges_pion[0]-1;
		echanges_pion[nb_echanges_pion]=0;
	}
	else 
	{
		ASSERT(nb_autres_coups>0);
		analysis=EXTEND_ANALYSE_AUTRES_COUPS;
	    #if COLLECT_STATS==1
		if (check==0)
		origine_coups_analysis_extend=OR_EXTEND_ECHANGE_PIECE;
		else
		origine_coups_analysis_extend=OR_EXTEND_ANALYSE_COUVERTURES_ECHECS;
	    #endif
		moveptr=&autres_coups[0]-1;
	}
} // fin check!=0
else // check==0
{
	if (hint_move!=0) 
	{
		moveptr=&hint_moves[0]-1;
		phase_generation=PRISES;
		F_utilisation_liste=true;
		analysis=EXTEND_ANALYSE_HINT_MOVE;
	    #if COLLECT_STATS==1
		   origine_coups_analysis_extend=OR_EXTEND_ANALYSE_HINT_HMOVE;
		#endif
	}
	else 
	{
		phase_generation=PRISES;
		numptr=&xw[debut_blanc]; num=debut_blanc;
		while(num<=fin_blanc)
		{
			if (numptr->nature!=0 && numptr->psptr->mobility_prises!=0)
			{ break; }
			numptr++; num++;
		}
		// Si pas de prises sortir en crant liste de prises vides
		if (num>fin_blanc) 
		{
			analysis=EXTEND_ANALYSE_PRISES_INSUFFISANTES_CANDIDATES_ECHEC; // Quand mme
			F_utilisation_liste=true; 
			prises_insuffisantes_candidates_echec[0]=0; // Liste vide
			moveptr=&prises_insuffisantes_candidates_echec[0]-1;
			F_pas_de_prises=true;
			goto QUIT_ANALYSIS;
		}
		analysis=EXTEND_ANALYSE_COUPS_FAVORABLES; // par principe
  	    #if COLLECT_STATS==1
		  origine_coups_analysis_extend=OR_EXTEND_ANALYSE_COUPS_FAVORABLES;
		#endif

		moveptr=numptr->psptr->moves2-1; // prises
		#if USE_CASES_ACTUALISATION==1
		F_piece_with_possibly_illegal_moves=bit(pieces_with_possible_illegal_moves,num);
		#endif
	}
}



QUIT_ANALYSIS:
return;
} // fin initialize




MOVE SEARCH::extend_get_move(void)
{
char cc;
//LEGALITY legality;
char arr;
char checking_nature;
char bilan_coup;
//char arr2;
short valeur_prise;
MOVE move;
char cat;
//bool F_move_takes_pawn;

NEXT_MOVE:
move=*(++moveptr);

if (move!=0)
{

PAS_UTILISATION_LISTE:
if (!F_utilisation_liste)
{

// Elimination du hint_move si dja jou
if (F_hint_move_has_been_played)
{
	if (SAME_MOVE(move,hint_move))
	{
		goto NEXT_MOVE;
	}
}

// Lgalit
#if USE_CASES_ACTUALISATION==1
if (F_piece_with_possibly_illegal_moves)
#endif
{
	cc=legal(num,move,num_roi,*lignemoins);
	if (cc!=1) // coup illgal
	{
		goto NEXT_MOVE;
	}
}

// Compteur de coups
nb_coups_liste++;

if (check==0 && phase_generation==PRISES)
{
valeur_prise=MOVE_CAPTURE_VALUE(move);
if (valpos+valeur_prise+POSITIONAL_PRUNING_MARGIN<=sup_alpha_value)
{

	if (F_extension_echecs)
	{
	prises_insuffisantes_candidates_echec[nb_prises_insuffisantes_candidates_echec++]=move;
	goto NEXT_MOVE;
	}




    #if USE_EXTEND_NODE_PV==1
	if (node_type==NODE_PV)
	{
        // Ces coups ne permettront de dpasser alpha que s'ils donnent chec
		prises_insuffisantes_candidates_echec[nb_prises_insuffisantes_candidates_echec++]=move;
		goto NEXT_MOVE;
	}
    #endif

	#if EXTEND_PRISES_INSUFFISANTES_CANDIDATES_ECHEC==0
		goto PRISE_INSUFFISANTE2;
	#endif

	#if EXTEND_PRISES_INSUFFISANTES_CANDIDATES_ECHEC==1
	#if ALLOW_SELECTIVE_CHECKS==1
    if (F_selective_checks && bit(selective_checks_piece_flag,MOVE_NUM_PIECE(move)))
	{
	prises_insuffisantes_candidates_echec[nb_prises_insuffisantes_candidates_echec++]=move;
	goto NEXT_MOVE;
	}
	#endif			
	#endif	

	#if EXTEND_PRISES_INSUFFISANTES_CANDIDATES_ECHEC_ONLY_ON_PV==1
		goto PRISE_INSUFFISANTE2;
	#endif

	// F_autorisation_echecs peut tre  false
	// du fait de la pruning margin
	// C'est pourquoi on se base sur un test sur xextendlevel
	if (!xextendlevel==xlevel && !F_extension_echecs) 
	{ goto PRISE_INSUFFISANTE2; }

	  	// Se justifie car mme si la prise est insuffisante
		// on pourrait pouvoir jouer  nouveau
		// Mais on ne joue le coup que s'il donne chec
		// Le test de SEE sera fait aprs
	    // Si ONLY_ON_PV, liminer la suite
		// Sinon, examiner ses conditions de recevabilit 

	    if (true
		#if RESTRICT_EXTEND_PRISES_INSUFFISANTES_CANDIDATES_ECHEC==1
		// On peut restreindre les prises insuffisantes candidates chec
		// aux extensions d'chec
            // ou aux chec selectifs
            // La plupart du temps, elles sont inutiles
		&& ( F_extension_echecs 
		    #if ALLOW_SELECTIVE_CHECKS==1
                   ||
			(F_selective_checks && bit(selective_checks_piece_flag,MOVE_NUM_PIECE(move)))
		    #endif
		   )
		#endif
		)
	   {
	   #if NO_EXTEND_QUEEN_CHECK_SACRIFICE==1
        if (MOVE_IS_QUEEN(move)) 
        {
	        #if USE_CASES_ACTUALISATION==1
                if (bit(cases_controlees_tour_adversaire,MOVE_ARR64(move)))
				{ goto PRISE_INSUFFISANTE2; }
			#else
				if (bilan_attaque_defense_avant_coup(move)==PERTE_MATERIELLE)
				{ goto PRISE_INSUFFISANTE2; }
			#endif
		}
		#endif
		prises_insuffisantes_candidates_echec[nb_prises_insuffisantes_candidates_echec++]=move;
		goto NEXT_MOVE;
 	   }
	 else // (valpos valeur prise) Elimination prise insuffisante
	{
		PRISE_INSUFFISANTE2:	
		// Avec valeur_prise
		#if RECHERCHE_DECOUVERTES==1 && DECOUVERTES_ON_EXTEND==1
		if (F_decouverte)
		{
			if (MOVE_NUM_PIECE(move)==decouverte_num)
			{
				return move;
			}
		}
		if (value<=alpha)
		{
		min_fail_low_value=sup(min_fail_low_value,valpos+valeur_prise+POSITIONAL_PRUNING_MARGIN); // fail-low - upperbound
		min_fail_low_value=inf(alpha,min_fail_low_value); 
		if (value<min_fail_low_value) value=min_fail_low_value;
		ASSERT(value<=alpha);
		}
		goto NEXT_MOVE;
		PRISE_INSUFFISANTE:
		// Prise insuffisante avec recalcul de la valeur prise
		// Rcupration dcouvertes
		// Cette rcupration est ncesssaire car on ne donne 
		// pas forcment chec
		#if RECHERCHE_DECOUVERTES==1 && DECOUVERTES_ON_EXTEND==1
		if (F_decouverte)
		{
			if (MOVE_NUM_PIECE(move)==decouverte_num)
			{
				return move;
			}
		}
		#endif
		if (value<=alpha)
		{
		min_fail_low_value=sup(min_fail_low_value,
			 valpos
			+MOVE_CAPTURE_VALUE(move)
			+POSITIONAL_PRUNING_MARGIN); // fail-low - upperbound
		min_fail_low_value=inf(alpha,min_fail_low_value); 
		if (value<min_fail_low_value) value=min_fail_low_value;
		ASSERT(value<=alpha);
		}
		goto NEXT_MOVE;
	}
}


// Traitement par catgorie
cat=MOVE_CATEGORY(move);
ASSERT(MOVE_CAPTURE(move)!='R');
// Mettre le coup en coup forant si la pice ou le pion de l'adversaire n'est pas protg
#if USE_CASES_ACTUALISATION==1
if (!bit(cases_controlees_adversaire,MOVE_ARR64(move)))
{
	if (MOVE_TAKES_PAWN(move))
	{
		cat=PRISE_FAVORABLE_PION;
	}
	else
	{
		cat=COUP_FORCANT;
	}
}
#endif
switch(cat)
{
case COUP_FORCANT :
	ASSERT(check==0);
	break;
case PROMOTION :
	ASSERT(check==0);
	//#if USE_CASES_ACTUALISATION==1
	//if (bit(cases_controlees_adversaire,MOVE_ARR64(move)))
	//{
	//	coups_declasses[nb_coups_declasses++]=move;
	//	goto NEXT_MOVE;
	//}
	//#endif
	bilan_coup=bilan_attaque_defense_avant_coup(move);
	if (bilan_coup==PERTE_MATERIELLE)
	{
		#if USE_EXTEND_NODE_PV==1
		if (node_type==NODE_PV)
	      {
		    coups_declasses[nb_coups_declasses++]=move;
			goto NEXT_MOVE;
		}
		#endif
		goto PRISE_INSUFFISANTE;
	}
	break;
case ECHANGE_PIECE :
	ECHANGE_PIECE:
	echanges_piece[nb_echanges_piece++]=move;
	goto NEXT_MOVE;
case PRISE_FAVORABLE_PION :
	// Concerne les prises de pions non protgs
	// Les prises de pions protgs sont valus
	// en prises de pice infrieure
	prises_pion_non_defendus[nb_prises_pion_non_defendus++]=move;
	goto NEXT_MOVE;
case ECHANGE_PION :
	echanges_pion[nb_echanges_pion++]=move;
	goto NEXT_MOVE;
case PRISE_PIECE_INFERIEURE :
	prises_piece_inferieure[nb_prises_piece_inferieure++]=move;
	goto NEXT_MOVE;
case PRISE_PION_PAR_PIECE :
	prises_piece_inferieure[nb_prises_piece_inferieure++]=move;
	goto NEXT_MOVE;
 default :
	 ASSERT(false);
	break;
}

}
} // Fin gnration hors liste
UTILISATION_LISTE:
// Pas de else pour les coups sous listes
// qui sont valides
;
}
else // (move==0)
{
if (!F_utilisation_liste)
{
	switch(phase_generation)
	{
	case PRISES : 
		numptr++; num++;
		while(num<=fin_blanc)
		{
			if (numptr->nature!=0 && numptr->psptr->mobility_prises!=0)
			{ break; }
			numptr++; num++;
		}
		if (num>fin_blanc) 
		{ goto CHANGEMENT_ANALYSIS; }
		moveptr=numptr->psptr->moves2-1; 
		#if USE_CASES_ACTUALISATION==1
		F_piece_with_possibly_illegal_moves=bit(pieces_with_possible_illegal_moves,num);
		#endif
		goto NEXT_MOVE;
	    break;
	case AUTRES_COUPS :
		ASSERT(F_autorisation_echecs); // parce que sinon pas d'autres coups
		// Et si check!=0 on est en analyse liste
		numptr++; num++;
		while(num<=fin_blanc)
		{
			if (numptr->nature==0)
			{ numptr++; num++; continue; }
			// Voir si la pice est autorise  donner des checs
			// Si extension d'chec, elle l'est
			#if RECHERCHE_DECOUVERTES==1 && DECOUVERTES_ON_EXTEND==1
			if (F_decouverte)
			{
				if (decouverte_num==num)
				{
					F_nextlevel_try_anticipated_extend_cutoff=false;
					goto DECOUVERTE3;
				}
			}
			#endif
			#if EXTEND_VERIFY_LINE==1
			// Si F_play_non_tactical_moves, pas de restriction
			if ( F_play_non_tactical_moves )
			{ goto PLAY_NON_TACTICAL_MOVES_3; }
			#endif
			if (check==0)
			{

					switch (numptr->nature)
					{
					case 'P' : 
						#if NO_EXTEND_PAWN_CHECKS==1
						#if USE_EXTEND_NODE_PV==1
						// Ces checs sont trop rares
						// Inutile de perdre du temps
						// if (node_type!=NODE_PV)
						{
						#endif
						if (!F_extension_echecs)
						{
							numptr++; num++;
							continue;
						}
						#if USE_EXTEND_NODE_PV==1
						}
						#endif
						#endif
						#if ALLOW_SELECTIVE_CHECKS==1
						if (F_selective_checks)
						{
							if (bit(selective_checks_piece_flag,num)==0)
							{
							numptr++; num++;
							continue;
							}
						}
						#endif
						#if ALLOW_COMPLEMENTARY_CHECKS==1
						if (check_filter!='A')
						{
							numptr++; num++;
							continue;
						}
						#endif
						// Position de pion
						#if USE_CASES_ACTUALISATION==0
						// perfectible
						if (abs(numptr->posx-s->roi_posx[other_wtm])>2)
						{
							numptr++; num++;
							continue;
						}
						if (abs(numptr->posx-s->roi_posx[other_wtm])>2)
						{
							numptr++; num++;
							continue;
						}
						#endif
						#if USE_CASES_ACTUALISATION==1
						if ((xcases_controlees_pion[other_wtm][s->roi_pos[other_wtm]]
							 &numptr->psptr->cases_actualisation)==0)
						{
							numptr++; num++;
							continue;
						}
						#endif
						break;
					case 'C' : 
						#if NO_EXTEND_KNIGHT_CHECKS==1
						#if USE_EXTEND_NODE_PV==1
						if (node_type!=NODE_PV)
						{
						#endif
						if (!F_extension_echecs)
						{
						numptr++; num++;
						continue;
						}
						#if USE_EXTEND_NODE_PV==1
						}
						#endif
						#endif
						#if ALLOW_SELECTIVE_CHECKS==1
						if (F_selective_checks)
						{
							if (bit(selective_checks_piece_flag,num)==0)
							{
							numptr++; num++;
							continue;
							}
						}
						#endif
						#if ALLOW_COMPLEMENTARY_CHECKS==1
						if (check_filter!='A' && check_filter!='C')
						{
							numptr++; num++;
							continue;
						}
						#endif
						// Plus efficace
						if (!(xcases_arrivee_cavalier[numptr->pos]&xcases_arrivee_cavalier[s->roi_pos[other_wtm]]))
						{
							numptr++; num++;
							continue;
						}
						// A CONSERVER 
						//#if USE_CASES_ACTUALISATION==1
						//if ((xcases_arrivee_cavalier[s->roi_pos[other_wtm]]&numptr->psptr->cases_actualisation)==0)
						//{
						//	numptr++; num++;
						//	continue;
						//}
						//#endif
						break;					
					case 'F' : 
						#if NO_EXTEND_BISHOP_CHECKS==1
						#if USE_EXTEND_NODE_PV==1
						if (node_type!=NODE_PV)
						{
						#endif
						if (!F_extension_echecs)
						{
						numptr++; num++;
						continue;
						}
						#if USE_EXTEND_NODE_PV==1
						}
						#endif
 					      #endif
						#if ALLOW_SELECTIVE_CHECKS==1
						if (F_selective_checks)
						{
							if (bit(selective_checks_piece_flag,num)==0)
							{
							numptr++; num++;
							continue;
							}
						}
						#endif
						#if ALLOW_COMPLEMENTARY_CHECKS==1
						if (check_filter!='A' && check_filter!='F') 
						{
							numptr++; num++;
							continue;
						}
						#endif
						#if USE_CASES_ACTUALISATION==0
						// Fou de la couleur du roi adverse ?
						if (M_2(X64(numptr->pos)+Y64(numptr->pos))
							!= M_2(X64(s->roi_pos[other_wtm])+Y64(s->roi_pos[other_wtm])))
						{
							numptr++; num++;
							continue;
						}
						#endif
						#if USE_CASES_ACTUALISATION==1
						if ((s->cases_echec_fou[xwtm]&numptr->psptr->cases_actualisation)==0)
						{
							numptr++; num++;
							continue;
						}
						#endif
						break;
					case 'T' : 					
						#if NO_EXTEND_ROOK_CHECKS==1
						#if USE_EXTEND_NODE_PV==1
						if (node_type!=NODE_PV)
						{
						#endif
						if (!F_extension_echecs)
						{
						numptr++; num++;
						continue;
						}
						#if USE_EXTEND_NODE_PV==1
						}
						#endif
					    #endif	
						#if ALLOW_SELECTIVE_CHECKS==1
						if (F_selective_checks)
						{
							if (bit(selective_checks_piece_flag,num)==0)
							{
							numptr++; num++;
							continue;
							}
						}
						#endif					
						#if ALLOW_COMPLEMENTARY_CHECKS==1
						if (check_filter!='A' && check_filter!='T')
						{
							numptr++; num++;
							continue;
						}
						#endif
						#if USE_CASES_ACTUALISATION==1
						if ((s->cases_echec_tour[xwtm]&numptr->psptr->cases_actualisation)==0)
						{
							numptr++; num++;
							continue;
						}
						#endif
						break;
					case 'D' : 					
						#if ALLOW_SELECTIVE_CHECKS==1
						if (F_selective_checks)
						{
							if (bit(selective_checks_piece_flag,num)==0)
							{
							numptr++; num++;
							continue;
							}
						}
						#endif
						#if ALLOW_COMPLEMENTARY_CHECKS==1
						if (check_filter!='A' && check_filter!='F' && check_filter!='T') 
						{ 
							numptr++; num++;
							continue;
							ASSERT(check_filter=='C');
						}
						#endif
						#if USE_CASES_ACTUALISATION==1
						if ((s->cases_echec_dame[xwtm]&numptr->psptr->cases_actualisation)==0)
						{
							numptr++; num++;
							continue;
						}
						#endif
						break;				
					case 'R' : 
						#if NO_EXTEND_KING_CHECKS==1
						#if USE_EXTEND_NODE_PV==1
						// toujours un de dcouverte de toute faon,
						// if (node_type!=NODE_PV)
						{
						#endif
						if (!F_extension_echecs)
						{
						numptr++; num++;
						continue;
						}
						#if USE_EXTEND_NODE_PV==1
						}
						#endif
						#endif
						#if ALLOW_SELECTIVE_CHECKS==1
						if (F_selective_checks)
						{
							// Pas de selective_checks pour le roi
							//if (bit(selective_checks_piece_flag,num)==0)
							//{
							numptr++; num++;
							continue;
							//}
						}
						#endif
						#if ALLOW_COMPLEMENTARY_CHECKS==1
						if (check_filter!='A') 
						{
							numptr++; num++;
							continue;
						}
						#endif
						break;
					default :
						break;
					}
			}
			DECOUVERTE3:
			#if EXTEND_VERIFY_LINE==1
			PLAY_NON_TACTICAL_MOVES_3:
			#endif
			if (numptr->psptr->mobility_others==0)
			{ numptr++; num++; continue; }
			// Pice trouve 
			break;
		}
		if (num>fin_blanc) 
		{ goto CHANGEMENT_ANALYSIS; }
		moveptr=numptr->psptr->moves-1; 
		#if USE_CASES_ACTUALISATION==1
		F_piece_with_possibly_illegal_moves=bit(pieces_with_possible_illegal_moves,num);
		#endif
		goto NEXT_MOVE;
		break;
	}
}
else // move==0 && F_utilisation_liste
{
	CHANGEMENT_ANALYSIS:
	switch(analysis)
	{
		case EXTEND_ANALYSE_HINT_MOVE :
			      F_hint_move_has_been_played=true;
				  #if COLLECT_STATS==1
				  origine_coups_analysis_extend=OR_EXTEND_ANALYSE_COUPS_FAVORABLES;
				  #endif
				  analysis=EXTEND_ANALYSE_COUPS_FAVORABLES;
				  if (check!=0)
				  {
					  // La liste de coups a dja t cre
					  if (nb_prises_favorables==0) 
					  {
						  goto CHANGEMENT_ANALYSIS;
					  }
					  moveptr=&prises_favorables[0];
	  				  move=*moveptr;
					  prises_favorables[nb_prises_favorables]=0;  // marquage fin
					  F_utilisation_liste=true;
					  goto JOUER_COUP;
				  }
				  else
				  {
					  phase_generation=PRISES;
				 	  numptr=&xw[debut_blanc]; num=debut_blanc;
			  		  while(num<=fin_blanc)
					  {
						if (numptr->nature!=0 && numptr->psptr->mobility_prises!=0)
						{ break; }
						numptr++; num++;
					  }
					  // Si pas d'autres coups, sortir
					  if (num>fin_blanc) goto CHANGEMENT_ANALYSIS;
					  moveptr=numptr->psptr->moves2-1;
 	  				  #if USE_CASES_ACTUALISATION==1
					  F_piece_with_possibly_illegal_moves=bit(pieces_with_possible_illegal_moves,num);
					  #endif
					  F_utilisation_liste=false;
					  goto NEXT_MOVE; // pour contrle de lgalit
				  }
			break;
		case EXTEND_ANALYSE_COUPS_FAVORABLES : 
				  analysis=EXTEND_PRISES_PIONS_NON_DEFENDUS;
  				  #if COLLECT_STATS==1
				  origine_coups_analysis_extend=OR_EXTEND_PRISES_PIONS_NON_DEFENDUS;
				  #endif

				  if (nb_prises_pion_non_defendus==0) 
				  {
					goto CHANGEMENT_ANALYSIS;
				  }
				  moveptr=&prises_pion_non_defendus[0]-1;
  				  move=*moveptr;
				  prises_pion_non_defendus[nb_prises_pion_non_defendus]=0;  // marquage fin
				  F_utilisation_liste=true;
				  goto NEXT_MOVE;
				

		case EXTEND_PRISES_PIONS_NON_DEFENDUS :
				  analysis=EXTEND_ANALYSE_ECHANGES_PIECE;
				  #if COLLECT_STATS==1
				  origine_coups_analysis_extend=OR_EXTEND_ECHANGE_PIECE;
				  #endif
				  if (nb_echanges_piece==0) 
				  {
					goto CHANGEMENT_ANALYSIS;
				  }
				  moveptr=&echanges_piece[0]-1;
				  move=*moveptr;
				  echanges_piece[nb_echanges_piece]=0;  // marquage fin
				  F_utilisation_liste=true;
				  goto NEXT_MOVE;

		case EXTEND_ANALYSE_ECHANGES_PIECE :
				  analysis=EXTEND_PRISES_PIECE_INFERIEURE;
				  if (nb_prises_piece_inferieure==0)
				  {
					goto CHANGEMENT_ANALYSIS;
				  }
  			      moveptr=&prises_piece_inferieure[0];
				  move=*moveptr;
				  prises_piece_inferieure[nb_prises_piece_inferieure]=0;  // marquage fin
				  F_utilisation_liste=true;
				  //goto JOUER_COUP;
				  goto EXAMINER_COUP;
				  break;






		case EXTEND_PRISES_PIECE_INFERIEURE :
   				  #if EXTEND_DISABLE_PAWN_EXCHANGES==1
				  if (check==0 && finale<FIN_DE_PARTIE_2) 
				  { goto SKIP_PAWN_EXCHANGES; }
				  #endif
                  analysis=EXTEND_ANALYSE_ECHANGES_PION;
				  if (nb_echanges_pion>0)
				  {
					  moveptr=&echanges_pion[0];
  					  move=*moveptr;
					  echanges_pion[nb_echanges_pion]=0;  // marquage fin
					  F_utilisation_liste=true;
					  //goto JOUER_COUP;
					  goto EXAMINER_COUP;
				  }
				  else 
				  {
					  goto CHANGEMENT_ANALYSIS;
				  }
				  break;




		case EXTEND_ANALYSE_ECHANGES_PION :
				  #if EXTEND_DISABLE_PAWN_EXCHANGES==1
					SKIP_PAWN_EXCHANGES:
				  #endif
				  if (!F_autorisation_echecs 
					  && check==0
					  #if EXTEND_PRISES_INSUFFISANTES_CANDIDATES_ECHEC==1
					  && nb_prises_insuffisantes_candidates_echec==0
					  #endif
					  && nb_coups_declasses==0
					  ) return 0;
				  
				  if (check!=0)
				  {
					  #if COLLECT_STATS==1
					  origine_coups_analysis_extend=OR_EXTEND_COUVERTURES_ECHECS;
					  #endif
					  // La liste de coups a dja t cre
					  analysis=EXTEND_ANALYSE_AUTRES_COUPS;
					  if (nb_autres_coups==0) 
					  {
						  goto CHANGEMENT_ANALYSIS;
					  }
					  moveptr=&autres_coups[0];
	  				  move=*moveptr;
					  autres_coups[nb_autres_coups]=0;  // marquage fin
					  F_utilisation_liste=true;
					  goto EXAMINER_COUP;
				  }
				  else // check==0
				  {
					  if (F_autorisation_echecs)
					  {
						  if (nb_prises_insuffisantes_candidates_echec>0)
						  {
							      // Coups dont-il faut vrifier qu'ils donnent 
								  #if COLLECT_STATS==1
								  origine_coups_analysis_extend=OR_EXTEND_ANALYSE_ECHECS;
								  #endif
								  // ou non chec
								  analysis=EXTEND_ANALYSE_PRISES_INSUFFISANTES_CANDIDATES_ECHEC;
								  moveptr=&prises_insuffisantes_candidates_echec[0];
	  							  move=*moveptr;
								  prises_insuffisantes_candidates_echec[nb_prises_insuffisantes_candidates_echec]=0;  // marquage fin
								  F_utilisation_liste=true;
								  goto EXAMINER_COUP;
						  }
					  	  #if COLLECT_STATS==1
						  origine_coups_analysis_extend=OR_EXTEND_ANALYSE_ECHECS;
						  #endif
						  #if EXTEND_USE_CASES_ACTUALISATION_FOR_CHECKS==0
						  analysis=EXTEND_ANALYSE_ECHECS;
						  phase_generation=AUTRES_COUPS;
				 		  numptr=&xw[debut_blanc]; num=debut_blanc;
			  			  while(num<=fin_blanc)
						  {
							if (numptr->nature==0)
							{numptr++; num++; continue;}


							#if EXTEND_VERIFY_LINE==1
							// Si F_play_non_tactical_moves, pas de restriction
							if ( F_play_non_tactical_moves )
							{ goto PLAY_NON_TACTICAL_MOVES_1; }
							#endif
							switch (numptr->nature)
							{
							case 'P' : 
								#if NO_EXTEND_PAWN_CHECKS==1
								#if USE_EXTEND_NODE_PV==1
								// if (node_type!=NODE_PV)
								{
								#endif
								if (!F_extension_echecs)
								{
									numptr++; num++;
									continue;
								}
								#if USE_EXTEND_NODE_PV==1
								}
								#endif
								#endif
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A')
								{
									numptr++; num++;
									continue;
								}
								#endif
								// Position de pion
								#if USE_CASES_ACTUALISATION==0
								// perfectible
								if (abs(numptr->posx-s->roi_posx[other_wtm])>2)
								{
									numptr++; num++;
									continue;
								}
								#endif
								#if USE_CASES_ACTUALISATION==1
								if ((xcases_controlees_pion[other_wtm][s->roi_pos[other_wtm]]
									 &numptr->psptr->cases_actualisation)==0)
								{
									numptr++; num++;
									continue;
								}
								#endif

								break;
							case 'C' : 
								#if NO_EXTEND_KNIGHT_CHECKS==1
								#if USE_EXTEND_NODE_PV==1
								if (node_type!=NODE_PV)
								{
								#endif
								if (!F_extension_echecs)
								{
								numptr++; num++;
								continue;
								}
								#if USE_EXTEND_NODE_PV==1
								}
								#endif
								#endif
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A' && check_filter!='C') 
								{
									numptr++; num++;
									continue;
								}
								#endif
								// Plus efficace
								if (!(xcases_arrivee_cavalier[numptr->pos]&xcases_arrivee_cavalier[s->roi_pos[other_wtm]]))
								{
									numptr++; num++;
									continue;
								}
								//#endif
								// A CONSERVER 
								//#if USE_CASES_ACTUALISATION==1
								//if ((xcases_arrivee_cavalier[s->roi_pos[other_wtm]]&numptr->psptr->cases_actualisation)==0)
								//{
								//	numptr++; num++;
								//	continue;
								//}
								//#endif
								break;					
							case 'F' : 
								#if NO_EXTEND_BISHOP_CHECKS==1
								#if USE_EXTEND_NODE_PV==1
								if (node_type!=NODE_PV)
								{
								#endif
								if (!F_extension_echecs)
								{
									numptr++; num++;
									continue;
								}
								#if USE_EXTEND_NODE_PV==1
								}
								#endif
								#endif
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A' && check_filter!='F') 
								{
									numptr++; num++;
									continue;
								}
								#endif
								#if USE_CASES_ACTUALISATION==0
								// Fou de la couleur du roi adverse ?
								if (M_2(X64(numptr->pos)+Y64(numptr->pos))
									!= M_2(X64(s->roi_pos[other_wtm])+Y64(s->roi_pos[other_wtm])))
								{
									numptr++; num++;
									continue;
								}
								#endif
								#if USE_CASES_ACTUALISATION==1
								if ((s->cases_echec_fou[xwtm]&numptr->psptr->cases_actualisation)==0)
								{
									numptr++; num++;
									continue;
								}
								#endif
								break;
							case 'T' : 					
								#if NO_EXTEND_ROOK_CHECKS==1
								#if USE_EXTEND_NODE_PV==1
								if (node_type!=NODE_PV)
								{
								#endif
								if (!F_extension_echecs)
								{
								numptr++; num++;
								continue;
								}
								#if USE_EXTEND_NODE_PV==1
								}
								#endif
								#endif						
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A' && check_filter!='T')
								{
									numptr++; num++;
									continue;
								}
								#endif
								#if USE_CASES_ACTUALISATION==1
								if ((s->cases_echec_tour[xwtm]&numptr->psptr->cases_actualisation)==0)
								{
									numptr++; num++;
									continue;
								}
								#endif
								break;
							case 'D' : 					
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter=='C') 
								{
									numptr++; num++;
									continue;
								}
								ASSERT(check_filter=='A' || check_filter=='T' || check_filter=='F');
								#endif
								#if USE_CASES_ACTUALISATION==1
								if ((s->cases_echec_dame[xwtm]&numptr->psptr->cases_actualisation)==0)
								{
									numptr++; num++;
									continue;
								}
								#endif
								break;				
							case 'R' : 
								#if NO_EXTEND_KING_CHECKS==1
								#if USE_EXTEND_NODE_PV==1
								//if (node_type!=NODE_PV)
								{
								#endif
								if (!F_extension_echecs)
								{
								numptr++; num++;
								continue;
								}
								#if USE_EXTEND_NODE_PV==1
								}
								#endif
								#endif
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									//if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A')
								{
									numptr++; num++;
									continue;
								}
								#endif
								break;
							default :
								break;
							}
							if (numptr->psptr->mobility_others==0)
							{ numptr++; num++; continue; }
							// Pice trouve 
							break;
						  }; // fin while
						  #if EXTEND_VERIFY_LINE==1
						  PLAY_NON_TACTICAL_MOVES_1:
						  #endif


						  // Si pas d'autres coups, voir s'il reste des coups dclasss
						  // et sinon sortir
						  if (num>fin_blanc) 
						  {
							  if (nb_coups_declasses>0) goto CHANGEMENT_ANALYSIS;
							  else return 0;
						  }
						  moveptr=numptr->psptr->moves-1;
						  #if USE_CASES_ACTUALISATION==1
						  F_piece_with_possibly_illegal_moves=bit(pieces_with_possible_illegal_moves,num);
						  #endif
						  F_utilisation_liste=false;
						  goto NEXT_MOVE; // pour contrle de lgalit

						  // EXTEND_USE_CASES_ACTUALISATION_FOR_CHECKS==0
						  #endif
						
						  #if EXTEND_USE_CASES_ACTUALISATION_FOR_CHECKS==1
						  analysis=EXTEND_ANALYSE_ECHECS;
						  // phase_generation=AUTRES_COUPS; la phase reste inchange
						  extend_creer_liste_echecs_sans_prises();
						  if (nb_echecs==0) 
						  {
							return 0;
						  }
						  moveptr=&echecs[0];
  						  move=*moveptr;
						  echecs[nb_echecs]=0;  // marquage fin
						  F_utilisation_liste=true;
						  goto EXAMINER_COUP;
						  // EXTEND_USE_CASES_ACTUALISATION_FOR_CHECKS==1
						  #endif
					  }
					  else // Pas d'autorisation checs
					  {
						    // Passer aux coups dclasss
						    // en sautant les autres coups
							if (nb_coups_declasses==0) 
							{
								return 0;
							}
		  				    analysis=EXTEND_ANALYSE_COUPS_DECLASSES;
							moveptr=&coups_declasses[0];
  							move=*moveptr;
							coups_declasses[nb_coups_declasses]=0;  // marquage fin
							F_utilisation_liste=true;
						    goto EXAMINER_COUP;
					  }
				  }
				  break;


		case EXTEND_ANALYSE_PRISES_INSUFFISANTES_CANDIDATES_ECHEC :
				  if (!F_autorisation_echecs 
					  && check==0
					  && nb_coups_declasses==0
					  ) return 0;
				  
					  ASSERT(check==0);
					  ASSERT(F_autorisation_echecs);
					  {

						  #if EXTEND_USE_CASES_ACTUALISATION_FOR_CHECKS==0
						  #if COLLECT_STATS==1
						  origine_coups_analysis_extend=OR_EXTEND_ANALYSE_ECHECS;
						  #endif
						  analysis=EXTEND_ANALYSE_ECHECS;
						  phase_generation=AUTRES_COUPS;
				 		  numptr=&xw[debut_blanc]; num=debut_blanc;
			  			  while(num<=fin_blanc)
						  {
							if (numptr->nature==0)
							{numptr++; num++; continue;}
							#if RECHERCHE_DECOUVERTES==1 && DECOUVERTES_ON_EXTEND==1
							if (F_decouverte && num==decouverte_num)
							{ 
								F_nextlevel_try_anticipated_extend_cutoff=false;
								goto DECOUVERTE2; 
							}
							#endif
							#if EXTEND_VERIFY_LINE==1
							// Si F_play_non_tactical_moves, pas de restriction
							if ( F_play_non_tactical_moves )
							{
							  goto PLAY_NON_TACTICAL_MOVES_2;
							}
							#endif

							switch (numptr->nature)
							{
							case 'P' : 
								#if NO_EXTEND_PAWN_CHECKS==1
								#if USE_EXTEND_NODE_PV==1
								//if (node_type!=NODE_PV)
								{
				                #endif
								if (!F_extension_echecs)
								{
									numptr++; num++;
									continue;
								}
								#if USE_EXTEND_NODE_PV==1
								}
								#endif
								#endif
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A')
								{
									numptr++; num++;
									continue;
								}
								#endif
								// Position de pion	
								#if USE_CASES_ACTUALISATION==0
								// perfectible
								if (abs(numptr->posx-s->roi_posx[other_wtm])>2)
								{
									numptr++; num++;
									continue;
								}
								#endif
								#if USE_CASES_ACTUALISATION==1
								if ((xcases_controlees_pion[other_wtm][s->roi_pos[other_wtm]]
								 &numptr->psptr->cases_actualisation)==0)
								{
									numptr++; num++;
									continue;
								}
								#endif
							 break;
							case 'C' : 
								#if NO_EXTEND_KNIGHT_CHECKS==1
								#if USE_EXTEND_NODE_PV==1
								if (node_type!=NODE_PV)
								{
								#endif
								if (!F_extension_echecs)
								{
								numptr++; num++;
								continue;
								}
								#if USE_EXTEND_NODE_PV==1
								}
								#endif
								#endif
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A' && check_filter!='C') 
								{
									numptr++; num++;
									continue;
								}
								#endif
								// Plus efficace
								if (!(xcases_arrivee_cavalier[numptr->pos]&xcases_arrivee_cavalier[s->roi_pos[other_wtm]]))
								{
									numptr++; num++;
									continue;
								}
								// A CONSERVER 
								//#if USE_CASES_ACTUALISATION==1
								//if ((xcases_arrivee_cavalier[s->roi_pos[other_wtm]]&numptr->psptr->cases_actualisation)==0)
								//{
								//	numptr++; num++;
								//	continue;
								//}
								//#endif
								break;					
							case 'F' : 
								#if NO_EXTEND_BISHOP_CHECKS==1
								#if USE_EXTEND_NODE_PV==1
								if (node_type!=NODE_PV)
								{
				                #endif
								if (!F_extension_echecs)
								{
								numptr++; num++;
								continue;
								}
								#if USE_EXTEND_NODE_PV==1
								}
								#endif
								#endif
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A' && check_filter!='F') 
								{
									numptr++; num++;
									continue;
								}
								#endif
								#if USE_CASES_ACTUALISATION==0
								// Fou de la couleur du roi adverse ?
								if (M_2(X64(numptr->pos)+Y64(numptr->pos))
									!= M_2(X64(s->roi_pos[other_wtm])+Y64(s->roi_pos[other_wtm])))
								{
									numptr++; num++;
									continue;
								}
								#endif
								#if USE_CASES_ACTUALISATION==1
								if ((s->cases_echec_fou[xwtm]&numptr->psptr->cases_actualisation)==0)
								{
									numptr++; num++;
									continue;
								}
								#endif
								break;
							case 'T' : 					
								#if NO_EXTEND_ROOK_CHECKS==1
								#if USE_EXTEND_NODE_PV==1
								if (node_type!=NODE_PV)
								{
				                #endif
								if (!F_extension_echecs)
								{
									numptr++; num++;
									continue;
								}
								#if USE_EXTEND_NODE_PV==1
								}
								#endif
								#endif						
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A' && check_filter!='T')
								{
									numptr++; num++;
									continue;
								}
								#endif
								#if USE_CASES_ACTUALISATION==1
								if ((s->cases_echec_tour[xwtm]&numptr->psptr->cases_actualisation)==0)
								{
									numptr++; num++;
									continue;
								}
								#endif
								break;
							case 'D' : 				
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A' && check_filter!='T' && check_filter!='C')
								{
									numptr++; num++;
									continue;
								}
								#endif
								#if USE_CASES_ACTUALISATION==1
								if ((s->cases_echec_dame[xwtm]&numptr->psptr->cases_actualisation)==0)
								{
									numptr++; num++;
									continue;
								}
								#endif
								break;				
							case 'R' : 
								#if NO_EXTEND_KING_CHECKS==1
								#if USE_EXTEND_NODE_PV==1
								//if (node_type!=NODE_PV)
								{
				                #endif
								if (!F_extension_echecs)
								{
									numptr++; num++;
									continue;
								}
								#if USE_EXTEND_NODE_PV==1
								}
								#endif
								#endif
								#if ALLOW_SELECTIVE_CHECKS==1
								if (F_selective_checks)
								{
									//if (bit(selective_checks_piece_flag,num)==0)
									{
										numptr++; num++;
										continue;
									}
								}
								#endif
								#if ALLOW_COMPLEMENTARY_CHECKS==1
								if (check_filter!='A')
								{
									numptr++; num++;
									continue;
								}
								#endif
								break;
							default :
								break;
							}
							DECOUVERTE2:
							#if EXTEND_VERIFY_LINE==1
							PLAY_NON_TACTICAL_MOVES_2:
							#endif

							if (numptr->psptr->mobility_others==0)
							{
								numptr++; num++;
								continue;
							}
							// Pice trouve 
							break;
						  }
						  // Si pas d'autres coups, voir s'il reste des coups dclasss
						  // et sinon sortir
						  if (num>fin_blanc) 
						  {
							  if (nb_coups_declasses>0) goto CHANGEMENT_ANALYSIS;
							  else return 0;
						  }
						  moveptr=numptr->psptr->moves-1;
						  #if USE_CASES_ACTUALISATION==1
						  F_piece_with_possibly_illegal_moves=bit(pieces_with_possible_illegal_moves,num);
						  #endif
						  F_utilisation_liste=false;
						  goto NEXT_MOVE; // pour contrle de lgalit
						  // EXTEND_USE_CASES_ACTUALISATION_FOR_CHECKS==0
						  #endif

						  #if EXTEND_USE_CASES_ACTUALISATION_FOR_CHECKS==1
						  analysis=EXTEND_ANALYSE_ECHECS;
						  // phase_generation=AUTRES_COUPS; la phase reste inchange
						  extend_creer_liste_echecs_sans_prises();
						  if (nb_echecs==0) 
						  {
							return 0;
						  }
						  moveptr=&echecs[0];
  						  move=*moveptr;
						  echecs[nb_echecs]=0;  // marquage fin
						  F_utilisation_liste=true;
						  goto EXAMINER_COUP;
						  #endif

				  }
				  break;

		case EXTEND_ANALYSE_ECHECS :
				  ASSERT(check==0);

				  if (nb_coups_declasses==0) 
				  {
					return 0;
				  }
				  analysis=EXTEND_ANALYSE_COUPS_DECLASSES;
				  #if COLLECT_STATS==1
				  if (check==0)
					  origine_coups_analysis_extend=OR_EXTEND_ANALYSE_COUPS_DECLASSES;
				  else
					  origine_coups_analysis_extend=OR_EXTEND_ANALYSE_COUVERTURES_ECHECS;
				  #endif
				  moveptr=&coups_declasses[0];
  				  move=*moveptr;
				  coups_declasses[nb_coups_declasses]=0;  // marquage fin
				  F_utilisation_liste=true;
				  goto JOUER_COUP;

		case EXTEND_ANALYSE_AUTRES_COUPS :

				  if (nb_coups_declasses==0) 
				  {
					return 0;
				  }
				  analysis=EXTEND_ANALYSE_COUPS_DECLASSES;
				  #if COLLECT_STATS==1
				  if (check==0)
					  origine_coups_analysis_extend=OR_EXTEND_ANALYSE_COUPS_DECLASSES;
				  else
					  origine_coups_analysis_extend=OR_EXTEND_ANALYSE_COUVERTURES_ECHECS;
				  #endif
				  moveptr=&coups_declasses[0];
  				  move=*moveptr;
				  coups_declasses[nb_coups_declasses]=0;  // marquage fin
				  F_utilisation_liste=true;
				  goto JOUER_COUP;
		
		case EXTEND_ANALYSE_COUPS_DECLASSES  :
				  return 0;
	}
}
} // fin move==0
	

EXAMINER_COUP:
switch (analysis)
{
	case EXTEND_ANALYSE_HINT_MOVE :

		return move;
		break;

	case EXTEND_ANALYSE_COUPS_FAVORABLES :

		#if _DEBUG==1
		cc=MOVE_CATEGORY(move);
		#endif
		//ASSERT(!(MOVE_TAKES_PAWN(move) && !MOVE_IS_FORCING(move)));

		// Prises favorables
		// Origine coups favorables
		#if COLLECT_STATS==1
		if ( check==0 )
		{ 
		if ( MOVE_CATEGORY(move)==COUP_FORCANT ) origine_coups_analysis_extend=OR_EXTEND_COUPS_FORCANTS;
		else if ( MOVE_CATEGORY(move)==PROMOTION ) origine_coups_analysis_extend=OR_EXTEND_PROMOTION;
		}
		#endif

		return move;
		break; 


	case EXTEND_ANALYSE_ECHANGES_PIECE :
		return move;
		break;

	case EXTEND_PRISES_PIONS_NON_DEFENDUS :
		return move;
		break;

	case EXTEND_PRISES_PIECE_INFERIEURE :

        // Elimination immdiate des prises de pices infrieures protges par
        // des pions (pour la dame en fait)

		// Les sacrifices de tours restent autoriss car les prises de
        // cavalier et de fou par des tours sont considrs 
		// comme des changes de pice

        // En dfinitive, tout ceci ne concerne que la dame
		// Et les prises de pions 
		#if ALLOW_EXTEND_ROOK_SACRIFICES==1
        ASSERT(MOVE_IS_QUEEN(move)||MOVE_TAKES_PAWN(move));
		#endif

        // Analyse du coup
        if (check==0)
		{
			// Un coup prenant  la dcouverte doit tre jou
			#if RECHERCHE_DECOUVERTES==1 && DECOUVERTES_ON_EXTEND==1
			if (F_decouverte)
			{
				if (MOVE_NUM_PIECE(move)==decouverte_num)
				return move;
			}
			#endif	
			#if USE_CASES_ACTUALISATION==0
            // Si USE_CASES_ACTUALISATION==1
            // le pawn_protected suffit
            if (is_pawn_protected(MOVE_ARR64(move),other_wtm))
            {
                #if USE_EXTEND_NODE_PV==1
                if (node_type==NODE_PV)
                {
                    coups_declasses[nb_coups_declasses++]=move;
                    goto NEXT_MOVE;
                }
                #endif
                valeur_prise=MOVE_CAPTURE_VALUE(move);
                min_fail_low_value=sup(min_fail_low_value,valpos+valeur_prise+POSITIONAL_PRUNING_MARGIN); // fail-low - upperbound
                min_fail_low_value=inf(alpha,min_fail_low_value);
                if (value<min_fail_low_value) value=min_fail_low_value;
                goto NEXT_MOVE;
            }
            #endif
            #if USE_CASES_ACTUALISATION==1
            if (MOVE_IS_QUEEN(move)) 
            {
                if (bit(cases_controlees_tour_adversaire,MOVE_ARR64(move)))
                {
					
					#if NO_EXTEND_QUEEN_TAKES_PROTECTED_PAWN==1
					if (MOVE_TAKES_PAWN(move))
					{ goto PRISE_INSUFFISANTE; }
					#endif
					

					#if USE_EXTEND_NODE_PV==1
					if (node_type==NODE_PV)
					{
						// Peut tre intressant
						// car les dfenseurs peuvent tre clous
						coups_declasses[nb_coups_declasses++]=move;
		                goto NEXT_MOVE;
					}
					#endif
					goto PRISE_INSUFFISANTE;
                }
            }
            // La case adverse est contrle par des pices adverses
            // sinon le coup aurait t un coup forant
            // mais il ne s'agit pas de pions ou de tours
            // Il peut s'agir d'une dame ou du roi adverse
            // Dans ce cas, un bilan coup est ncessaire
            #endif

			// Pawn protected ?
			ASSERT(wtm==xwtm && other_wtm==1-xwtm);
			if (is_pawn_protected(MOVE_ARR64(move),other_wtm))
			{
	  		    #if USE_EXTEND_NODE_PV==1
			    if (node_type==NODE_PV)
			    {
					 // Il faut jouer le coup
					 // car les  dfenseurs peuvent tre clous
				     coups_declasses[nb_coups_declasses++]=move;
		             goto NEXT_MOVE;
			    }
                      #endif
			    goto PRISE_INSUFFISANTE;
            }

            bilan_coup=bilan_attaque_defense_avant_coup(move);
            if (bilan_coup==PERTE_MATERIELLE)
            {
                #if USE_EXTEND_NODE_PV==1
                if (node_type==NODE_PV)
                {
                    coups_declasses[nb_coups_declasses++]=move;
                    goto NEXT_MOVE;
                }
                #endif
		    goto PRISE_INSUFFISANTE;
            }
        }
        return move;
        break;

	case EXTEND_ANALYSE_ECHANGES_PION :
		return move;
		break;

	case EXTEND_ANALYSE_PRISES_INSUFFISANTES_CANDIDATES_ECHEC :
		goto PLAY_IF_GIVECHECK;
		break;

	case EXTEND_ANALYSE_ECHECS : // Echecs, couvertures d'checs


			ASSERT(check==0);	
			ASSERT(F_autorisation_echecs);
			ASSERT(xlevel==xextendlevel || F_selective_checks || F_extension_echecs);
			#if EXTEND_USE_CASES_ACTUALISATION_FOR_CHECKS==1
				// L'analyse d'opportunit de la dcouverte est dja faite
				if (MOVE_NUM_PIECE(move)==decouverte_num) goto DECOUVERTE;
				
				// Sinon chec
				// On a retard l'tape SEE qui consomme du temps
				goto BILAN_COUP;
			#endif


			// Analyse classique qu'il faut conserver 
			// mme en EXTEND_USE_CASES_ACTUALISATION_FOR_CHECKS==1
			// pour le cas des prises insuffisantes envoyes par 
			// goto PLAY_IF_GIVECHECK;

			PLAY_IF_GIVECHECK:
		
		    #if ALLOW_SELECTIVE_CHECKS==1
			if (F_selective_checks)
			{
				// Vrifier qu'on ne joue que l'chec slectif
				// permettant de capturer une pice adverse
				ASSERT(bit(selective_checks_piece_flag,MOVE_NUM_PIECE(move))
					||analysis==EXTEND_ANALYSE_PRISES_INSUFFISANTES_CANDIDATES_ECHEC);
				if (bit(selective_checks_arrival_squares,MOVE_ARR64(move)))
				{ goto PLAY_SELECTIVE_CHECK; }
				else 
				{
				  goto NEXT_MOVE;
				}
			}		
		    #endif

			// Vrification 
			// le coup donne-t-il chec ?
			// Les sous-promotions seulement si elles donnent chec
			checking_nature=MOVE_NATURE(move);
			if (MOVE_IS_PROMOTION(move))
			{ 
				checking_nature=MOVE_PROMOTION(move);
			}


			#if RECHERCHE_DECOUVERTES==1 && DECOUVERTES_ON_EXTEND==1
			//ASSERT(xboard[MOVE_DEP64(move)].num_piece=numptr->num_piece);
			while (F_decouverte)
			{
				if (MOVE_NUM_PIECE(move)==decouverte_num)
				{	ASSERT(decouverte_move_nature==xw[decouverte_num].nature);
					ASSERT(decouverte_move_nature==MOVE_NATURE(move));
					if (MOVE_IS_CAPTURE(move)) goto DECOUVERTE;
					switch(decouverte_move_nature)
					{
						case 'C' :
							 if (SEARCH::givecheck_cavalier(MOVE_ARR64(move))) goto DECOUVERTE;
							 cases_menacees_decouverte=xcases_arrivee_cavalier[MOVE_ARR64(move)];
							 
							 break;
						case 'F' :
							 if (givecheck64(s->cases_echec_fou[wtm],MOVE_ARR64(move))) goto DECOUVERTE;
							 cases_menacees_decouverte=xcases_arrivee_fou[MOVE_ARR64(move)];
							 break;
						case 'T' : 
							 if (givecheck64(s->cases_echec_tour[wtm],MOVE_ARR64(move))) goto DECOUVERTE;
							 cases_menacees_decouverte=xcases_arrivee_tour[MOVE_ARR64(move)];
							 break;
						case 'R' : 
							 cases_menacees_decouverte=xcases_arrivee_roi[MOVE_ARR64(move)];
							 break;
						case 'P' : 
							 cases_menacees_decouverte=xcases_controlees_pion[wtm][MOVE_ARR64(move)];
							 break;
					      case 'D' : 
							ASSERT(1==2);
							break;
						default : 
							cases_menacees_decouverte=0;
							break;
					}

					if (decouverte_move_nature=='P' || decouverte_move_nature=='R')
					{
					   if (abs(MOVE_INC64(move))
						!=abs(xinc64_from64_to64[MOVE_DEP64(move)][s->roi_pos[wtm]]))
					   {
							if (cases_cibles_decouverte & cases_menacees_decouverte)
							{ goto DECOUVERTE; }
							else break;
					   }
					}
					else
					{
	  						if (cases_cibles_decouverte & cases_menacees_decouverte)
							{ goto DECOUVERTE; }
							else break;
					}
				}
			    break;
			}; // fin while
			#endif
			


			// Limitation des checs de pices mineures
			// pourquoi un test NO_EXTEND_<PIECE>_CHECKS  ce niveau l ?
			// On pourrait venir des prises sur checs infrieures
			// Le tests NO_EXTEND sont ici en pratique inutiles
			// car un coup n'arrive ici en gnral qu'avec le droit de donner chec

			// Note le filtrage F_play_non_tactical_moves est fait plus haut
			

			arr=MOVE_ARR64(move);
			switch(checking_nature)
			{
				case 'P' : 
					#if NO_EXTEND_PAWN_CHECKS==1
					ASSERT(MOVE_IS_TACTICAL(move)||F_extension_echecs||F_at_least_some_non_tactical_moves_played
						|| node_type==NODE_PV);
					#endif
					#if ALLOW_COMPLEMENTARY_CHECKS==1
					ASSERT(check_filter=='A');
					#endif
					if (!givecheck_pion(arr,wtm)) 
					{ 
						/*
						#if EXTEND_VERIFY_LINE==1
						if (F_play_non_tactical_moves)
						{
							ASSERT(F_extension_echecs);
							coups_declasses[nb_coups_declasses++];
							goto NEXT_MOVE;
						}
						#endif
						*/
					if (MOVE_IS_CAPTURE(move))
					{
						goto PRISE_INSUFFISANTE;
					}
					goto NEXT_MOVE;
				   	}	
				   	break;
				case 'C' :
					#if NO_EXTEND_KNIGHT_CHECKS==1
					ASSERT(MOVE_IS_TACTICAL(move)||F_extension_echecs||F_at_least_some_non_tactical_moves_played);
					#endif
					#if ALLOW_COMPLEMENTARY_CHECKS==1
					ASSERT(check_filter=='A' || check_filter=='C');
					#endif
					if (!SEARCH::givecheck_cavalier(arr))
					{ 
						/*
						#if EXTEND_VERIFY_LINE==1
						if (F_play_non_tactical_moves)
						{
							ASSERT(F_extension_echecs);
							coups_declasses[nb_coups_declasses++];
							goto NEXT_MOVE;
						}
						#endif
						*/
						if (MOVE_IS_CAPTURE(move))
						{
							goto PRISE_INSUFFISANTE;					
						}
						goto NEXT_MOVE;
				   	}	
			        break;
				case 'F' : 
					#if NO_EXTEND_BISHOP_CHECKS==1
					ASSERT(MOVE_IS_TACTICAL(move)||F_extension_echecs||F_at_least_some_non_tactical_moves_played
						||node_type==NODE_PV
						);
					#endif
					#if ALLOW_COMPLEMENTARY_CHECKS==1
					ASSERT(check_filter=='A' || check_filter=='F');
					#endif
					if (!givecheck64(s->cases_echec_fou[wtm],arr) ) 
					{ 
						/*
						#if EXTEND_VERIFY_LINE==1
						if (F_play_non_tactical_moves)
						{
							ASSERT(F_extension_echecs);
							coups_declasses[nb_coups_declasses++];
							goto NEXT_MOVE;
						}
						#endif
						*/
						if (MOVE_IS_CAPTURE(move))
						{
							goto PRISE_INSUFFISANTE;
						}
						goto NEXT_MOVE;
				   	}	
					break;
				case 'T' : 
					#if NO_EXTEND_ROOK_CHECKS==1
					ASSERT(MOVE_IS_TACTICAL(move)||F_extension_echecs||F_at_least_some_non_tactical_moves_played
						|| node_type==NODE_PV
						);
					#endif
					#if ALLOW_COMPLEMENTARY_CHECKS==1
					ASSERT(check_filter=='A' || check_filter=='T');
					#endif
					if (!givecheck64(s->cases_echec_tour[wtm],arr) ) 
					{ 
						/*
						#if EXTEND_VERIFY_LINE==1
						if (F_play_non_tactical_moves)
						{
							ASSERT(F_extension_echecs);
							coups_declasses[nb_coups_declasses++];
							goto NEXT_MOVE;
						}
						#endif
						*/
						if (MOVE_IS_CAPTURE(move))
						{
							goto PRISE_INSUFFISANTE;
						}
						goto NEXT_MOVE;
				   	}	
				    break;
			    case 'D' : 
					#if ALLOW_COMPLEMENTARY_CHECKS==1
					ASSERT(check_filter!='C');
					#endif

 				      if (!givecheck64(s->cases_echec_dame[wtm],arr) ) 
					{ 
						/*
						#if EXTEND_VERIFY_LINE==1
						if (F_play_non_tactical_moves)
						{
							ASSERT(F_extension_echecs);
							coups_declasses[nb_coups_declasses++];
							goto NEXT_MOVE;
						}
						#endif
						*/
						if (MOVE_IS_CAPTURE(move))
						{
							goto PRISE_INSUFFISANTE;
						}
						goto NEXT_MOVE;
				   	}	
					#if CHECK_EXTENSION_PLAY_ALL_CHECKS==1
					if (F_extension_echecs)
					{ break; }
					#endif
					#if NO_EXTEND_QUEEN_CHECK_SACRIFICE==1
			            #if USE_CASES_ACTUALISATION==1
				        if (bit(cases_controlees_tour_adversaire,MOVE_ARR64(move)))
						{ 
							if (MOVE_IS_CAPTURE(move))
							{ 
								#if USE_EXTEND_NODE_PV==1
								// Ne jouer la prise que si NODE_PV
								if (node_type!=NODE_PV) 
								{ 
									{ goto PRISE_INSUFFISANTE; }
								}
								#else
								{ goto PRISE_INSUFFISANTE; }
								#endif
							}
							else
							{ goto NEXT_MOVE; }
						}
					#else
						if (bilan_attaque_defense_avant_coup(move)==PERTE_MATERIELLE)
						{ 
							if (MOVE_IS_CAPTURE(move))
							{ 
								#if NO_EXTEND_QUEEN_TAKES_PROTECTED_PAWN==1
								if (MOVE_TAKES_PAWN(move))
								{ goto PRISE_INSUFFISANTE; }
								#endif
								#if USE_EXTEND_NODE_PV==1
								// Ne jouer la prise que si NODE_PV
								if (node_type!=NODE_PV) 
								{ goto PRISE_INSUFFISANTE; }		
								#else
								{ goto PRISE_INSUFFISANTE; }
								#endif
							}
							else
								{ goto NEXT_MOVE; }
						}
					#endif
					#endif
					break;
				case 'R' :  		
					#if NO_EXTEND_KING_CHECKS==1
					ASSERT(MOVE_IS_TACTICAL(move)||F_extension_echecs||F_at_least_some_non_tactical_moves_played|| node_type==NODE_PV);
					#endif
					#if ALLOW_COMPLEMENTARY_CHECKS==1
					ASSERT(check_filter=='A');
					#endif
					if (!SEARCH::givecheck_roi(move) ) 
					{ 
						/*
						#if EXTEND_VERIFY_LINE==1
						if (F_play_non_tactical_moves)
						{
							ASSERT(F_extension_echecs);
							coups_declasses[nb_coups_declasses++];
							goto NEXT_MOVE;
						}
						#endif
						*/
					if (MOVE_IS_CAPTURE(move))
					{
						goto PRISE_INSUFFISANTE;
					}
					goto NEXT_MOVE;
				   	}	
				break;
			      default : break;
			}
				
			
			// Bilan attaque-dfense
			//cc=MOVE_IS_TACTICAL(move);
			//cc=MOVE_IS_CAPTURE(move);
			ASSERT(!MOVE_IS_TACTICAL(move) && check==0
				|| (MOVE_IS_PROMOTION(move) && MOVE_PROMOTION(move)!='D')
				|| analysis==EXTEND_ANALYSE_PRISES_INSUFFISANTES_CANDIDATES_ECHEC
				|| num==decouverte_num
				);

		    // Les deux catgories ont dja t tudies
			#if _DEBUG
			cat=MOVE_CATEGORY(move);
			#endif
			
			// Bilan
			// il est utile pour les prises insuffisantes candidates chec
			// et pour les autres coups
			ASSERT(!MOVE_IS_CAPTURE(move) 
				|| analysis==EXTEND_ANALYSE_PRISES_INSUFFISANTES_CANDIDATES_ECHEC
				);


			
			// En node PV, on analyse tous les checs
			// mme avec perte matrielle
			// mme les prises insuffisantes
			//  l'exception des sacrifices de dame
			// Les sacrifices de dame, quant  eux
			// sont exclus ci-dessus
			BILAN_COUP:
			if (  true
				#if USE_EXTEND_NODE_PV==1
				&& node_type!=NODE_PV
				#endif
				#if CHECK_EXTENSION_PLAY_ALL_CHECKS==1
				&& F_extension_echecs==false
			    #endif
			    )
			{

			// Tous les doivent avoir leur bilan calcul
 			// car il peut y avoir des prises insuffisantes
			// Donc soit favorables (on les joue), soit dfavorables (on ne les joue pas)
			if (bilan_attaque_defense_avant_coup(move)==PERTE_MATERIELLE)
			{ 
			{
				//ASSERT(!MOVE_IS_CAPTURE(move));
				goto NEXT_MOVE;
		   	}
			}
			}

			#if RECHERCHE_DECOUVERTES==1 && DECOUVERTES_ON_EXTEND==1
			DECOUVERTE:
			#endif
			#if ALLOW_SELECTIVE_CHECKS
			PLAY_SELECTIVE_CHECK:
			#endif

			// Le coup donne chec et il faut actualiser au niveau suprieur
			F_nextlevel_try_anticipated_extend_cutoff=false;

			#if ALLOW_SELECTIVE_CHECKS==1
			ASSERT(F_extension_echecs||MOVE_IS_QUEEN(move)
				  || xlevel==xextendlevel || F_selective_checks);
            #endif
			#if ALLOW_COMPLEMENTARY_CHECKS==1
					ASSERT(F_extension_echecs||MOVE_IS_QUEEN(move)
				  || xlevel==xextendlevel || (check_filter!='A' && s->complementary_check_status[other_wtm]!=0
				                                && s->complementary_checks[wtm]<=2
										));
			#endif
			#if  ALLOW_COMPLEMENTARY_CHECKS==0 && ALLOW_SELECTIVE_CHECKS==0
			      ASSERT(F_extension_echecs||MOVE_IS_QUEEN(move)
				  || xlevel==xextendlevel);
			#endif
		

			#if COLLECT_STATS==1
		    if (check==0)
				origine_coups_analysis_extend=OR_EXTEND_ECHEC;
			else
				origine_coups_analysis_extend=OR_EXTEND_ANALYSE_COUVERTURES_ECHECS;
			#endif
		#endif
		// Fin choix classique des checs
	
		return move;		
		break; // Fin ECHECS


	case EXTEND_ANALYSE_AUTRES_COUPS :
		// Autres coups (sous chec ou analyse coups non tactiques)

		#if EXTEND_VERIFY_LINE==0
		ASSERT(check!=0);
		#endif
		
		#if EXTEND_VERIFY_LINE==1
		// Si F_play_non_tactical_moves, pas de restriction
		ASSERT(!(check==0 && !F_play_non_tactical_moves);
		if (check==0 && F_play_non_tactical_moves)
		{
			ASSERT(check==0);
			#if _DEBUG==1
			F_at_least_some_non_tactical_moves_played=true;
			#endif

			// On joue tous les coups de bilan neutre
			if (bilan_attaque_defense_avant_coup(move)==PERTE_MATERIELLE)
			{	 
				goto NEXT_MOVE;
			}
			else
			{
				return move;
			}
		}
		#endif

		// Sinon, couvertures d'checs  jouer
		// Lgalit dja vrifie
		return move;
		break;


	case EXTEND_ANALYSE_COUPS_DECLASSES :
		// Coups dclasss 
		return move;
		break;
	default : 
		ASSERT(false);
		break;

} // fin switch

JOUER_COUP:
return move;
} // fin get_move